﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace P15_2
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        //соотношение сторон окна вывода
        float aspectRatio;

        //Матрицы
        Matrix viewMatrix;
        Matrix projMatrix;
        Matrix worldMatrix;
        //Модель
        Model ball;
        
        // Позиция и вращение камеры
        Vector3 camPos = new Vector3(0, 0, -50);
        float camRotation;

        // Направление камеры
        Vector3 cameraDirection = new Vector3(0, 0, 1);

        // Установка скорости поворота и передвижения
        float rotationSpeed = 0.01f;
        float forwardSpeed = 1.0f;

        //Угол зрения камеры
        float viewAngle = MathHelper.ToRadians(45.0f);
        GraphicsDeviceManager graphics;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            //Загрузка модели
            ball = Content.Load<Model>("ball");
            //Установка мировой матрицы
            worldMatrix = Matrix.Identity;
            //Соотношение сторон
            aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width /
(float)graphics.GraphicsDevice.Viewport.Height;
     
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            //Получить состояние клавиатуры
            KeyboardState keyboardState = Keyboard.GetState();

            //Поворот влево
            if (keyboardState.IsKeyDown(Keys.Left))
            {
                 camRotation += rotationSpeed;
            }
            //Поворот вправо
            if (keyboardState.IsKeyDown(Keys.Right))
            {
                camRotation -= rotationSpeed;
            }

            //Переедвижение вперед
            if (keyboardState.IsKeyDown(Keys.Up))
            {
                //Создаем матрицу поворота
                Matrix forwardMov = Matrix.CreateRotationY(camRotation);
                //Создаем новый вектор, содержащий скорость движения
                Vector3 v = new Vector3(0, 0, forwardSpeed);
                //Трансформируем матрицу в вектор
                v = Vector3.Transform(v, forwardMov);
                //модифицируем позицию таким образом, чтобы
                //она соответствовала новым данным
                camPos.Z += v.Z;
                camPos.X += v.X;
            }
            //Передвижение назад
            if (keyboardState.IsKeyDown(Keys.Down))
            {
                Matrix forwardMovement = Matrix.CreateRotationY(camRotation);
                Vector3 v = new Vector3(0, 0, -forwardSpeed);
                v = Vector3.Transform(v, forwardMovement);
                camPos.Z += v.Z;
                camPos.X += v.X;
            }

            //Уменьшение угла обзора камеры
            if (keyboardState.IsKeyDown(Keys.W))
            {
                viewAngle -= MathHelper.ToRadians(1.0f);
            }
            //Увеличение угла обзора камеры
            if (keyboardState.IsKeyDown(Keys.S))
            {
                viewAngle += MathHelper.ToRadians(1.0f);
            }
            //Если новый угол обзора вышел за дозволенные пределы
            //изменяем его
            if (viewAngle > MathHelper.ToRadians(180.0f)) viewAngle = MathHelper.ToRadians(179.9f);
            if (viewAngle < MathHelper.ToRadians(0.0f)) viewAngle = MathHelper.ToRadians(0.1f);

            Matrix rotationMatrix = Matrix.CreateRotationY(camRotation);

            // Вектор направления камеры
            Vector3 cameraTransformed = Vector3.Transform(cameraDirection, rotationMatrix);

            // Вычисляем положение камеры
            Vector3 cameraLookat = camPos + cameraTransformed;

            // Устанавливаем матрицу вида
            viewMatrix = Matrix.CreateLookAt(camPos, cameraLookat, new Vector3(0.0f, 1.0f, 0.0f));
            //Устанавливаем проекционную матрицу
            projMatrix = Matrix.CreatePerspectiveFieldOfView(viewAngle, aspectRatio, 1.0f, 1000.0f);

            base.Update(gameTime);
        }
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            foreach (ModelMesh mesh in ball.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.LightingEnabled = true;
                    effect.EnableDefaultLighting();
                    effect.Projection = projMatrix;
                    effect.View = viewMatrix;
                    effect.World = worldMatrix;

                }
                mesh.Draw();
            }

            base.Draw(gameTime);
        }
    }
}
